@@ -1,5 +1,5 @@
 package Log::Log4perl::Layout::JSON;
-$Log::Log4perl::Layout::JSON::VERSION = '0.58';
+$Log::Log4perl::Layout::JSON::VERSION = '0.59';
 # ABSTRACT: Layout a log message as a JSON hash, including MDC data
 
 use 5.010;
@@ -71,6 +71,7 @@ use Class::Tiny {
 
         return $content;
     },
+    maxkb => undef,
     canonical => 0,
     include_mdc => 0,
     exclude_message => 0,
@@ -104,6 +105,14 @@ sub BUILD { ## no critic (RequireArgUnpacking)
         $self->field($field);
     }
 
+    if ($args->{maxkb}) {
+        my $maxkb = delete $args->{maxkb};
+
+        $self->_build_maxkb_values($maxkb);
+
+        $self->maxkb($maxkb);
+    }
+
     # Optionally override encoding from ascii to utf8
     if (my $arg = $args->{utf8}) {
         delete $args->{utf8};
@@ -174,6 +183,43 @@ sub _build_field_values {
     }
 }
 
+sub _build_maxkb_values {
+    my($self, $maxkb_hash) = @_;
+
+    while (my($key, $value) = each %$maxkb_hash) {
+        if (exists $value->{value} && !ref($value->{value})) {
+            $maxkb_hash->{$key} = $value->{value};
+        }
+        else {
+            $self->_build_maxkb_values($value)
+        }
+    }
+}
+
+sub _truncate_value {
+    my($self, $value_ref, $maxkb) = @_;
+
+    return if !$value_ref || !${$value_ref};
+    return if !$maxkb;
+
+    if (ref(${$value_ref}) eq 'HASH' && ref($maxkb) eq 'HASH') {
+        while (my($maxkb_key, $maxkb_val) = each %$maxkb) {
+            next unless ${$value_ref}->{$maxkb_key};
+
+            $self->_truncate_value(\${$value_ref}->{$maxkb_key}, $maxkb_val)
+        }
+    }
+    elsif (!ref(${$value_ref}) && !ref($maxkb)) {
+        my $len = length ${$value_ref};
+        my $maxb = $maxkb * 1024;
+
+        return if $len < $maxb;
+
+        my $trunc_marker = "...[truncated, was $len chars total]...";
+        substr(${$value_ref}, $maxb - length($trunc_marker)) = $trunc_marker;
+    }
+}
+
 
 sub render {
     my($self, $message, $category, $priority, $caller_level) = @_;
@@ -215,6 +261,17 @@ sub render {
         }
     }
 
+    if (my $maxkb = $self->maxkb) {
+        for ( my $i = 0; $i < $#fields; $i += 2) {
+            my($k, $v_ref) = ( $fields[$i], \$fields[$i+1] );
+
+            my $field_maxkb = $maxkb->{$k};
+            next unless $field_maxkb;
+
+            $self->_truncate_value($v_ref, $field_maxkb);
+        }
+    }
+
     my $max_json_length = $self->max_json_length_kb * 1024;
     my @dropped;
     my $json;
@@ -255,7 +312,7 @@ sub render {
                     $len = length $encoded;
                 }
                 else {
-                    $len = length $v;
+                    $len = length($v) || 0;
                 }
                 next if $len <= $max_json_length/2;
 
@@ -322,7 +379,7 @@ Log::Log4perl::Layout::JSON - Layout a log message as a JSON hash, including MDC
 
 =head1 VERSION
 
-version 0.58
+version 0.59
 
 =head1 SYNOPSIS
 
@@ -336,6 +393,9 @@ Example configuration:
     log4perl.appender.Example.layout.field.sub = %M{1}
     log4perl.appender.Example.layout.include_mdc = 1
 
+    # Optional truncation of specific fields
+    log4perl.appender.Example.layout.maxkb.message = 2
+
     # Note: Appender option!
     # log4perl.appender.Example.warp_message = 0
 
@@ -406,6 +466,17 @@ Include the data in the Log::Log4perl::MDC hash.
 
 See also L</name_for_mdc>.
 
+=head2 maxkb
+
+Use this name with the field name to specify a maximum length for a specific
+field. For example:
+
+    log4perl.appender.Example.maxkb.message = 2
+
+Will truncate message if it is more than 2048 bytes in length.  Truncated
+message will have a marker at the end like
+C<...[truncated, was $len chars total]...>
+
 =head2 name_for_mdc
 
 Use this name as the key in the JSON hash for the contents of MDC data
